//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once

#include <azac_api_c_common.h>
#include <vision_api_c_common.h>

/*! \cond INTERNAL */

namespace Azure {
namespace AI {
namespace Core {
namespace Details {

class LifetimeBinder
{
public:
    LifetimeBinder(void * context, AZAC_REFCOUNT_CLEAR_FN releaseFn)
    {
        AZAC_THROW_ON_FAIL(::vision_refcount_create(context, releaseFn, &m_handle));
    }

    LifetimeBinder(const LifetimeBinder& other)
    {
        AZAC_THROW_ON_FAIL(::vision_refcount_add_ref(other.m_handle, &m_handle));
    }

    LifetimeBinder(LifetimeBinder&& other)
    {
        m_handle = other.m_handle;
        other.m_handle = AZAC_HANDLE_INVALID;
    }

    LifetimeBinder& operator=(const LifetimeBinder& other)
    {
        Clear(m_handle);
        AZAC_THROW_ON_FAIL(::vision_refcount_add_ref(other.m_handle, &m_handle));
        return *this;
    }

    LifetimeBinder& operator=(LifetimeBinder&& other)
    {
        Clear(m_handle);
        m_handle = other.m_handle;
        other.m_handle = AZAC_HANDLE_INVALID;
        return *this;
    }

    ~LifetimeBinder()
    {
        Clear(m_handle);
    }

private:
    static void Clear(AZAC_HANDLE& handle)
    {
        if (handle != AZAC_HANDLE_INVALID)
        {
            ::vision_refcount_release(handle);
            handle = AZAC_HANDLE_INVALID;
        }
    }

    AZAC_HANDLE m_handle;
};

} } } } // Azure::AI::Core::Details

/*! \endcond */
