//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once

#include <cstdarg>
#include <type_traits>
#include <utility>

#include <azac_api_cxx_common.h>
#include <azac_api_cxx_details_property_collection.h>
#include <azac_api_cxx_details_string_helpers.h>
#include <vision_api_c_errors.h>
#include <vision_api_cxx_nullable.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace Core {
namespace Details {

/// <summary>
/// Helper struct to check the types of objects
/// </summary>
template<typename T, typename U>
struct is_container_of
{
    /// <summary>
    /// True if a list of objects belong to a particular type. Used with FrameSetWriter::WriteFrameSet.
    ///
    /// Not supported for ImageAnalyzer.
    /// </summary>
    static constexpr bool value = std::is_same<typename std::decay_t<T>::value_type, U>::value &&
                                  std::is_same<typename std::decay_t<T>::iterator, decltype(std::declval<T>().begin())>::value &&
                                  std::is_same<typename std::decay_t<T>::iterator, decltype(std::declval<T>().end())>::value;
};

} } } } } // Azure::AI::Vision::Core::Details

