//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once
#include <azac_api_cxx_details_property_collection.h>
#include <vision_api_cxx_common.h>
#include <vision_api_c_event_args.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace Core {
namespace Details {

/// <summary>
/// Base class for all vision event arguments
/// </summary>
template<typename TPropKey>
class EventArgsBase : public AI::Core::Details::PropertyCollection<TPropKey>
{
public:

    using BaseProperties = AI::Core::Details::PropertyCollection<TPropKey>;

    ~EventArgsBase()
    {
        if (vision_event_args_handle_is_valid(m_eventArgs))
        {
            ::vision_event_args_handle_release(m_eventArgs);
            m_eventArgs = AZAC_HANDLE_INVALID;
        }
    };

protected:
    /*! \cond PROTECTED */
    explicit EventArgsBase(AZAC_HANDLE eventArgs) :
        AI::Core::Details::PropertyCollection<TPropKey>(eventArgs, [](auto handle, auto* properties) { return vision_event_args_properties_handle_get(handle, properties); }),
        m_eventArgs(eventArgs)
    {
    }

    explicit operator AZAC_HANDLE() const { return m_eventArgs; }
    /*! \endcond */

private:

    AZAC_DISABLE_DEFAULT_CTORS(EventArgsBase);

    AZAC_HANDLE m_eventArgs;
};

} } } } } // Azure::AI::Vision::Core::Details
