//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once
#include <vision_api_cxx_common.h>
#include <vision_api_cxx_details_event_args_base.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace Core {
namespace Details {

/// <summary>
/// Base class for getting results out of vision events
/// </summary>
template<typename TResult, typename TPropKey>
class ResultEventArgsBase : private EventArgsBase<TPropKey>
{
private:

    template<typename Target> using ProtectedAccess = ::Azure::AI::Core::Details::ProtectedAccess<Target>;

public:

    std::shared_ptr<TResult> GetResult() const
    {
        AZAC_HANDLE result = AZAC_HANDLE_INVALID;
        auto eventArgs = ProtectedAccess<EventArgsBase<TPropKey>>::HandleFromPtr((EventArgsBase<TPropKey>*)this);
        AZAC_THROW_ON_FAIL(vision_event_args_result_handle_get(eventArgs, &result));
        return ProtectedAccess<TResult>::FromHandle(result);
    }

protected:
    /*! \cond PROTECTED */
    explicit ResultEventArgsBase(AZAC_HANDLE eventArgs) : EventArgsBase<TPropKey>(eventArgs)
    {
    }
    /*! \endcond */

private:

    AZAC_DISABLE_DEFAULT_CTORS(ResultEventArgsBase);
};

} } } } } // Azure::AI::Vision::Core::Details
