//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once
#include <vision_api_cxx_common.h>
#include <vision_api_cxx_details_session_event_args_base.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace Core {
namespace Details {

/// <summary>
/// Base class for vision session stopped event arguments
/// </summary>
template<typename TStoppedReason, typename TPropKey>
class SessionStoppedEventArgsBase : public SessionEventArgsBase<TPropKey>
{
public:

    TStoppedReason GetReason(TStoppedReason min, TStoppedReason max) const
    {
        auto property = SessionEventArgsBase<TPropKey>::Get("session.stopped.reason", "0");
        auto value = std::stoi(property.c_str());
        AZAC_IFTRUE_THROW_HR(value < (int)min || value > (int)max, AZAC_ERR_INVALID_RESULT_REASON);
        return (TStoppedReason)value;
    }

protected:
    /*! \cond PROTECTED */
    explicit SessionStoppedEventArgsBase(AZAC_HANDLE eventArgs) : SessionEventArgsBase<TPropKey>(eventArgs)
    {
    }
    /*! \endcond */

private:

    AZAC_DISABLE_DEFAULT_CTORS(SessionStoppedEventArgsBase);
};

} } } } } // Azure::AI::Vision::Core::Details
