//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once
#include <vision_api_cxx_common.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace Core {
namespace Session {

/// <summary>
/// Configuration options for the current vision session.
///
/// Not use by ImageAnalyzer.
/// </summary>
enum class VisionSessionOption
{
    /// <summary>
    /// Advanced Initialization Parameters.
    /// </summary>
    AdvancedInitializationParameters = 0,

    /// <summary>
    /// Application Name.
    /// </summary>
    ApplicationName = 1,

    /// <summary>
    /// Application Context.
    /// </summary>
    ApplicationContext = 2,

    /// <summary>
    /// The deployment Id string to use as configuration for a predefined operation.
    /// </summary>
    DeploymentId = 3,

    /// <summary>
    /// The operation Id string to use for define the operation and its configurations
    /// </summary>
    OperationId = 4,

    /// <summary>
    /// The json string configuration for the required operation
    /// </summary>
    OperationConfigurationJson = 6,

    /// <summary>
    /// The path for a configuration file to be used in the recognizer
    /// </summary>
    OperationConfigurationFilePath = 7

    // TODO: TFS#3664464 - Vision: What VisionSessionOption's should we have?
};

} } } } } // Azure::AI::Vision::Core::Session

PRIVATE_PROPERTY_COLLECTION_STATICS(Azure::AI::Vision::Core::Session::VisionSessionOption, "session")
