//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once
#include <vision_api_cxx_common.h>
#include <vision_api_cxx_details_result_event_args_base.h>
#include <vision_api_cxx_details_event_args_base.h>
#include <vision_api_cxx_session_result.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace Core {
namespace Session {

/// <summary>
/// Represents inference recognition event arguments.
/// </summary>
class SessionResultEventArgs : private Core::Details::ResultEventArgsBase<SessionResult, int>
{
private:

    using BaseResultEventArgs = Core::Details::ResultEventArgsBase<SessionResult, int>;

public:

    /// <summary>
    /// Gets the Result of an AI inference operation
    /// </summary>
    /// <returns>The SessionResult wrapped inside a std::shared_ptr</returns>
    std::shared_ptr<SessionResult> GetResult() const { return BaseResultEventArgs::GetResult(); }

protected:
    /*! \cond PROTECTED */
    static std::shared_ptr<SessionResultEventArgs> FromHandle(AZAC_HANDLE handle)
    {
        auto ptr = new SessionResultEventArgs(handle);
        return std::shared_ptr<SessionResultEventArgs>(ptr);
    }

    explicit SessionResultEventArgs(AZAC_HANDLE eventArgs) : ResultEventArgsBase(eventArgs)
    {
    }
    /*! \endcond */

private:

    AZAC_DISABLE_DEFAULT_CTORS(SessionResultEventArgs);
};

} } } } } // Azure::AI::Vision::Core::Session
