//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once
#include <vision_api_cxx_common.h>
#include <vision_api_cxx_details_session_stopped_event_args_base.h>
#include <vision_api_cxx_session_stopped_reason.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace Core {
namespace Session {

/// <summary>
/// Represents inference processing stopping.
/// </summary>
class SessionStoppedEventArgs : private Core::Details::SessionStoppedEventArgsBase<SessionStoppedReason, int>
{
private:

    using BaseEventArgs = Core::Details::SessionStoppedEventArgsBase<SessionStoppedReason, int>;

public:

    /// <summary>
    /// Gets the unique Session ID from which this SessionResult originated.
    /// </summary>
    /// <returns>
    /// The Session ID string.
    /// </returns>
    std::string GetSessionId() const { return BaseEventArgs::GetSessionId(); }

    /// <summary>
    /// Gets the unique Session ID from which this SessionResult originated.
    /// </summary>
    /// <returns>
    /// The Session ID string.
    /// </returns>
    template<class T = std::string> 
    AI::Core::Details::enable_if_w_or_string_t<T> GetSessionId() const { return BaseEventArgs::GetSessionId<T>(); }

    /// <summary>
    /// Gets the SessionStoppedReason for generation of this result.
    /// </summary>
    /// <returns>
    /// The session stopped reason
    /// </returns>
    SessionStoppedReason GetReason() const { return BaseEventArgs::GetReason(SessionStoppedReason::Error, SessionStoppedReason::StopRequested); }

protected:
    /*! \cond PROTECTED */
    static std::shared_ptr<SessionStoppedEventArgs> FromHandle(AZAC_HANDLE handle)
    {
        auto ptr = new SessionStoppedEventArgs(handle);
        return std::shared_ptr<SessionStoppedEventArgs>(ptr);
    }

    explicit SessionStoppedEventArgs(AZAC_HANDLE eventArgs) : SessionStoppedEventArgsBase(eventArgs)
    {
    }

    explicit operator AZAC_HANDLE() const { return AI::Core::Details::ProtectedAccess<EventArgsBase>::HandleFromConstPtr(this); }
    /*! \endcond */

private:

    AZAC_DISABLE_DEFAULT_CTORS(SessionStoppedEventArgs);
};

} } } } } // Azure::AI::Vision::Core::Session
